@@ -304,9 +304,14 @@ static void STDMETHODCALLTYPE d3d10_device_DrawInstanced(ID3D10Device1 *iface,
         UINT instance_vertex_count, UINT instance_count,
         UINT start_vertex_location, UINT start_instance_location)
 {
-    FIXME("iface %p, instance_vertex_count %u, instance_count %u, start_vertex_location %u,\n"
-            "\tstart_instance_location %u stub!\n", iface, instance_vertex_count, instance_count,
+    struct d3d10_device *device = impl_from_ID3D10Device(iface);
+
+    TRACE("iface %p, instance_vertex_count %u, instance_count %u, start_vertex_location %u, "
+            "start_instance_location %u.\n", iface, instance_vertex_count, instance_count,
             start_vertex_location, start_instance_location);
+
+    wined3d_device_draw_primitive_instanced(device->wined3d_device, start_vertex_location,
+            instance_vertex_count, start_instance_location, instance_count);
 }
 
 static void STDMETHODCALLTYPE d3d10_device_GSSetConstantBuffers(ID3D10Device1 *iface,
@@ -462,13 +467,53 @@ static void STDMETHODCALLTYPE d3d10_device_OMSetBlendState(ID3D10Device1 *iface,
         ID3D10BlendState *blend_state, const FLOAT blend_factor[4], UINT sample_mask)
 {
     struct d3d10_device *device = impl_from_ID3D10Device(iface);
+    const D3D10_BLEND_DESC *desc;
 
-    TRACE("iface %p, blend_state %p, blend_factor [%f %f %f %f], sample_mask 0x%08x.\n",
+    TRACE("iface %p, blend_state %p, blend_factor {%.8e %.8e %.8e %.8e}, sample_mask 0x%08x.\n",
             iface, blend_state, blend_factor[0], blend_factor[1], blend_factor[2], blend_factor[3], sample_mask);
 
-    device->blend_state = unsafe_impl_from_ID3D10BlendState(blend_state);
+    if (blend_factor[0] != 1.0f || blend_factor[1] != 1.0f || blend_factor[2] != 1.0f || blend_factor[3] != 1.0f)
+        FIXME("Ignoring blend factor {%.8e %.8e %.8e %.8e}.\n",
+                blend_factor[0], blend_factor[1], blend_factor[2], blend_factor[3]);
     memcpy(device->blend_factor, blend_factor, 4 * sizeof(*blend_factor));
-    device->sample_mask = sample_mask;
+    wined3d_device_set_render_state(device->wined3d_device, WINED3D_RS_MULTISAMPLEMASK, sample_mask);
+    if (!(device->blend_state = unsafe_impl_from_ID3D10BlendState(blend_state)))
+    {
+        wined3d_device_set_render_state(device->wined3d_device, WINED3D_RS_ALPHABLENDENABLE, FALSE);
+        wined3d_device_set_render_state(device->wined3d_device,
+                WINED3D_RS_COLORWRITEENABLE, D3D10_COLOR_WRITE_ENABLE_ALL);
+        wined3d_device_set_render_state(device->wined3d_device,
+                WINED3D_RS_COLORWRITEENABLE1, D3D10_COLOR_WRITE_ENABLE_ALL);
+        wined3d_device_set_render_state(device->wined3d_device,
+                WINED3D_RS_COLORWRITEENABLE2, D3D10_COLOR_WRITE_ENABLE_ALL);
+        wined3d_device_set_render_state(device->wined3d_device,
+                WINED3D_RS_COLORWRITEENABLE3, D3D10_COLOR_WRITE_ENABLE_ALL);
+        return;
+    }
+
+    desc = &device->blend_state->desc;
+    /* glSampleCoverage() */
+    if (desc->AlphaToCoverageEnable)
+        FIXME("Ignoring AlphaToCoverageEnable %#x.\n", desc->AlphaToCoverageEnable);
+    /* glEnableIndexedEXT(GL_BLEND, ...) */
+    FIXME("Per-rendertarget blend enable not implemented.\n");
+    wined3d_device_set_render_state(device->wined3d_device, WINED3D_RS_ALPHABLENDENABLE, desc->BlendEnable[0]);
+    wined3d_device_set_render_state(device->wined3d_device, WINED3D_RS_SRCBLEND, desc->SrcBlend);
+    wined3d_device_set_render_state(device->wined3d_device, WINED3D_RS_DESTBLEND, desc->DestBlend);
+    wined3d_device_set_render_state(device->wined3d_device, WINED3D_RS_BLENDOP, desc->BlendOp);
+    wined3d_device_set_render_state(device->wined3d_device, WINED3D_RS_SEPARATEALPHABLENDENABLE, TRUE);
+    wined3d_device_set_render_state(device->wined3d_device, WINED3D_RS_SRCBLENDALPHA, desc->SrcBlendAlpha);
+    wined3d_device_set_render_state(device->wined3d_device, WINED3D_RS_DESTBLENDALPHA, desc->DestBlendAlpha);
+    wined3d_device_set_render_state(device->wined3d_device, WINED3D_RS_BLENDOPALPHA, desc->BlendOpAlpha);
+    FIXME("Color mask > 3 not implemented.\n");
+    wined3d_device_set_render_state(device->wined3d_device,
+            WINED3D_RS_COLORWRITEENABLE, desc->RenderTargetWriteMask[0]);
+    wined3d_device_set_render_state(device->wined3d_device,
+            WINED3D_RS_COLORWRITEENABLE1, desc->RenderTargetWriteMask[1]);
+    wined3d_device_set_render_state(device->wined3d_device,
+            WINED3D_RS_COLORWRITEENABLE2, desc->RenderTargetWriteMask[2]);
+    wined3d_device_set_render_state(device->wined3d_device,
+            WINED3D_RS_COLORWRITEENABLE3, desc->RenderTargetWriteMask[3]);
 }
 
 static void STDMETHODCALLTYPE d3d10_device_OMSetDepthStencilState(ID3D10Device1 *iface,
@@ -1105,7 +1150,7 @@ static void STDMETHODCALLTYPE d3d10_device_OMGetBlendState(ID3D10Device1 *iface,
     if ((*blend_state = device->blend_state ? &device->blend_state->ID3D10BlendState_iface : NULL))
         ID3D10BlendState_AddRef(*blend_state);
     memcpy(blend_factor, device->blend_factor, 4 * sizeof(*blend_factor));
-    *sample_mask = device->sample_mask;
+    *sample_mask = wined3d_device_get_render_state(device->wined3d_device, WINED3D_RS_MULTISAMPLEMASK);
 }
 
 static void STDMETHODCALLTYPE d3d10_device_OMGetDepthStencilState(ID3D10Device1 *iface,
@@ -1235,32 +1280,122 @@ static UINT STDMETHODCALLTYPE d3d10_device_GetExceptionMode(ID3D10Device1 *iface
 static HRESULT STDMETHODCALLTYPE d3d10_device_GetPrivateData(ID3D10Device1 *iface,
         REFGUID guid, UINT *data_size, void *data)
 {
-    FIXME("iface %p, guid %s, data_size %p, data %p stub!\n",
+    IDXGIDevice *dxgi_device;
+    HRESULT hr;
+
+    TRACE("iface %p, guid %s, data_size %p, data %p.\n",
             iface, debugstr_guid(guid), data_size, data);
 
-    return E_NOTIMPL;
+    if (FAILED(hr = ID3D10Device1_QueryInterface(iface, &IID_IDXGIDevice, (void **)&dxgi_device)))
+        return hr;
+    hr = IDXGIDevice_GetPrivateData(dxgi_device, guid, data_size, data);
+    IDXGIDevice_Release(dxgi_device);
+
+    return hr;
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_device_SetPrivateData(ID3D10Device1 *iface,
         REFGUID guid, UINT data_size, const void *data)
 {
-    FIXME("iface %p, guid %s, data_size %u, data %p stub!\n",
+    IDXGIDevice *dxgi_device;
+    HRESULT hr;
+
+    TRACE("iface %p, guid %s, data_size %u, data %p.\n",
             iface, debugstr_guid(guid), data_size, data);
 
-    return E_NOTIMPL;
+    if (FAILED(hr = ID3D10Device1_QueryInterface(iface, &IID_IDXGIDevice, (void **)&dxgi_device)))
+        return hr;
+    hr = IDXGIDevice_SetPrivateData(dxgi_device, guid, data_size, data);
+    IDXGIDevice_Release(dxgi_device);
+
+    return hr;
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_device_SetPrivateDataInterface(ID3D10Device1 *iface,
         REFGUID guid, const IUnknown *data)
 {
-    FIXME("iface %p, guid %s, data %p stub!\n", iface, debugstr_guid(guid), data);
+    IDXGIDevice *dxgi_device;
+    HRESULT hr;
 
-    return E_NOTIMPL;
+    TRACE("iface %p, guid %s, data %p.\n", iface, debugstr_guid(guid), data);
+
+    if (FAILED(hr = ID3D10Device1_QueryInterface(iface, &IID_IDXGIDevice, (void **)&dxgi_device)))
+        return hr;
+    hr = IDXGIDevice_SetPrivateDataInterface(dxgi_device, guid, data);
+    IDXGIDevice_Release(dxgi_device);
+
+    return hr;
 }
 
 static void STDMETHODCALLTYPE d3d10_device_ClearState(ID3D10Device1 *iface)
 {
-    FIXME("iface %p stub!\n", iface);
+    static const float blend_factor[] = {1.0f, 1.0f, 1.0f, 1.0f};
+    struct d3d10_device *device = impl_from_ID3D10Device(iface);
+    unsigned int i;
+
+    TRACE("iface %p.\n", iface);
+
+    wined3d_device_set_vertex_shader(device->wined3d_device, NULL);
+    for (i = 0; i < D3D10_COMMONSHADER_SAMPLER_SLOT_COUNT; ++i)
+    {
+        wined3d_device_set_vs_sampler(device->wined3d_device, i, NULL);
+    }
+    for (i = 0; i < D3D10_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT; ++i)
+    {
+        wined3d_device_set_vs_resource_view(device->wined3d_device, i, NULL);
+    }
+    for (i = 0; i < D3D10_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT; ++i)
+    {
+        wined3d_device_set_vs_cb(device->wined3d_device, i, NULL);
+    }
+    wined3d_device_set_geometry_shader(device->wined3d_device, NULL);
+    for (i = 0; i < D3D10_COMMONSHADER_SAMPLER_SLOT_COUNT; ++i)
+    {
+        wined3d_device_set_gs_sampler(device->wined3d_device, i, NULL);
+    }
+    for (i = 0; i < D3D10_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT; ++i)
+    {
+        wined3d_device_set_gs_resource_view(device->wined3d_device, i, NULL);
+    }
+    for (i = 0; i < D3D10_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT; ++i)
+    {
+        wined3d_device_set_gs_cb(device->wined3d_device, i, NULL);
+    }
+    wined3d_device_set_pixel_shader(device->wined3d_device, NULL);
+    for (i = 0; i < D3D10_COMMONSHADER_SAMPLER_SLOT_COUNT; ++i)
+    {
+        wined3d_device_set_ps_sampler(device->wined3d_device, i, NULL);
+    }
+    for (i = 0; i < D3D10_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT; ++i)
+    {
+        wined3d_device_set_ps_resource_view(device->wined3d_device, i, NULL);
+    }
+    for (i = 0; i < D3D10_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT; ++i)
+    {
+        wined3d_device_set_ps_cb(device->wined3d_device, i, NULL);
+    }
+    for (i = 0; i < D3D10_IA_VERTEX_INPUT_RESOURCE_SLOT_COUNT; ++i)
+    {
+        wined3d_device_set_stream_source(device->wined3d_device, i, NULL, 0, 0);
+    }
+    wined3d_device_set_index_buffer(device->wined3d_device, NULL, WINED3DFMT_UNKNOWN);
+    wined3d_device_set_vertex_declaration(device->wined3d_device, NULL);
+    wined3d_device_set_primitive_type(device->wined3d_device, WINED3D_PT_UNDEFINED);
+    for (i = 0; i < D3D10_SIMULTANEOUS_RENDER_TARGET_COUNT; ++i)
+    {
+        wined3d_device_set_rendertarget_view(device->wined3d_device, i, NULL, FALSE);
+    }
+    wined3d_device_set_depth_stencil_view(device->wined3d_device, NULL);
+    ID3D10Device1_OMSetDepthStencilState(iface, NULL, 0);
+    ID3D10Device1_OMSetBlendState(iface, NULL, blend_factor, D3D10_DEFAULT_SAMPLE_MASK);
+    ID3D10Device1_RSSetViewports(iface, 0, NULL);
+    ID3D10Device1_RSSetScissorRects(iface, 0, NULL);
+    ID3D10Device1_RSSetState(iface, NULL);
+    for (i = 0; i < D3D10_SO_BUFFER_SLOT_COUNT; ++i)
+    {
+        wined3d_device_set_stream_output(device->wined3d_device, i, NULL, 0);
+    }
+    wined3d_device_set_predication(device->wined3d_device, NULL, FALSE);
 }
 
 static void STDMETHODCALLTYPE d3d10_device_Flush(ID3D10Device1 *iface)
@@ -1275,7 +1410,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateBuffer(ID3D10Device1 *iface,
     struct d3d10_buffer *object;
     HRESULT hr;
 
-    FIXME("iface %p, desc %p, data %p, buffer %p partial stub!\n", iface, desc, data, buffer);
+    TRACE("iface %p, desc %p, data %p, buffer %p.\n", iface, desc, data, buffer);
 
     object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
     if (!object)
@@ -1308,18 +1443,17 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateTexture2D(ID3D10Device1 *ifa
         const D3D10_TEXTURE2D_DESC *desc, const D3D10_SUBRESOURCE_DATA *data,
         ID3D10Texture2D **texture)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *device = impl_from_ID3D10Device(iface);
     struct d3d10_texture2d *object;
     HRESULT hr;
 
-    FIXME("iface %p, desc %p, data %p, texture %p partial stub!\n", iface, desc, data, texture);
+    TRACE("iface %p, desc %p, data %p, texture %p.\n", iface, desc, data, texture);
 
     object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
     if (!object)
         return E_OUTOFMEMORY;
 
-    hr = d3d10_texture2d_init(object, This, desc);
-    if (FAILED(hr))
+    if (FAILED(hr = d3d10_texture2d_init(object, device, desc, data)))
     {
         WARN("Failed to initialize texture, hr %#x.\n", hr);
         HeapFree(GetProcessHeap(), 0, object);
@@ -1347,8 +1481,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateTexture3D(ID3D10Device1 *ifa
     if (!object)
         return E_OUTOFMEMORY;
 
-    hr = d3d10_texture3d_init(object, device, desc);
-    if (FAILED(hr))
+    if (FAILED(hr = d3d10_texture3d_init(object, device, desc, data)))
     {
         WARN("Failed to initialize texture, hr %#x.\n", hr);
         HeapFree(GetProcessHeap(), 0, object);
@@ -2067,13 +2200,6 @@ static const struct ID3D10MultithreadVtbl d3d10_multithread_vtbl =
     d3d10_multithread_GetMultithreadProtected,
 };
 
-static void STDMETHODCALLTYPE d3d10_subresource_destroyed(void *parent) {}
-
-static const struct wined3d_parent_ops d3d10_subresource_parent_ops =
-{
-    d3d10_subresource_destroyed,
-};
-
 /* IWineDXGIDeviceParent IUnknown methods */
 
 static inline struct d3d10_device *device_from_dxgi_device_parent(IWineDXGIDeviceParent *iface)
@@ -2348,7 +2474,6 @@ HRESULT d3d10_device_init(struct d3d10_device *device, void *outer_unknown)
     device->blend_factor[1] = 1.0f;
     device->blend_factor[2] = 1.0f;
     device->blend_factor[3] = 1.0f;
-    device->sample_mask = D3D10_DEFAULT_SAMPLE_MASK;
 
     if (wine_rb_init(&device->depthstencil_states, &d3d10_depthstencil_state_rb_ops) == -1)
     {
