@@ -356,7 +356,9 @@ static void clean_mesh_builder_data(struct d3drm_mesh_builder *mesh_builder)
     }
     mesh_builder->nb_materials = 0;
     HeapFree(GetProcessHeap(), 0, mesh_builder->materials);
+    mesh_builder->materials = NULL;
     HeapFree(GetProcessHeap(), 0, mesh_builder->material_indices);
+    mesh_builder->material_indices = NULL;
 }
 
 static HRESULT WINAPI d3drm_mesh_builder2_QueryInterface(IDirect3DRMMeshBuilder2 *iface, REFIID riid, void **out)
@@ -1078,7 +1080,6 @@ HRESULT load_mesh_data(IDirect3DRMMeshBuilder3 *iface, IDirectXFileData *pData,
     This->nb_vertices = *(DWORD*)ptr;
     This->nb_faces = *(DWORD*)(ptr + sizeof(DWORD) + This->nb_vertices * sizeof(D3DVECTOR));
     faces_vertex_idx_size = size - sizeof(DWORD) - This->nb_vertices * sizeof(D3DVECTOR) - sizeof(DWORD);
-    faces_vertex_idx_ptr = (DWORD*)(ptr + sizeof(DWORD) + This->nb_vertices * sizeof(D3DVECTOR) + sizeof(DWORD));
 
     TRACE("Mesh: nb_vertices = %d, nb_faces = %d, faces_vertex_idx_size = %d\n", This->nb_vertices, This->nb_faces, faces_vertex_idx_size);
 
