@@ -39,7 +39,6 @@ static const struct IDirectXFileBinaryVtbl IDirectXFileBinary_Vtbl;
 static const struct IDirectXFileDataVtbl IDirectXFileData_Vtbl;
 static const struct IDirectXFileDataReferenceVtbl IDirectXFileDataReference_Vtbl;
 static const struct IDirectXFileEnumObjectVtbl IDirectXFileEnumObject_Vtbl;
-static const struct IDirectXFileObjectVtbl IDirectXFileObject_Vtbl;
 static const struct IDirectXFileSaveObjectVtbl IDirectXFileSaveObject_Vtbl;
 
 static HRESULT IDirectXFileDataReferenceImpl_Create(IDirectXFileDataReferenceImpl** ppObj);
@@ -620,7 +619,7 @@ static HRESULT WINAPI IDirectXFileDataImpl_GetId(IDirectXFileData* iface, LPGUID
   if (!pGuid)
     return DXFILEERR_BADVALUE;
 
-  memcpy(pGuid, &This->pobj->class_id, 16);
+  *pGuid = This->pobj->class_id;
 
   return DXFILE_OK;
 }
@@ -668,7 +667,7 @@ static HRESULT WINAPI IDirectXFileDataImpl_GetType(IDirectXFileData* iface, cons
   if (!pguid)
     return DXFILEERR_BADVALUE;
 
-  memcpy(&guid, &This->pobj->type, 16);
+  guid = This->pobj->type;
   *pguid = &guid;
 
   return DXFILE_OK;
@@ -887,7 +886,7 @@ static HRESULT WINAPI IDirectXFileDataReferenceImpl_GetId(IDirectXFileDataRefere
   if (!pGuid)
     return DXFILEERR_BADVALUE;
 
-  memcpy(pGuid, &This->ptarget->class_id, 16);
+  *pGuid = This->ptarget->class_id;
 
   return DXFILE_OK;
 }
