@@ -59,6 +59,9 @@ struct FvfToDecl
 
 #define DDRAW_STRIDE_ALIGNMENT  8
 
+#define DDRAW_WINED3D_FLAGS     (WINED3D_LEGACY_DEPTH_BIAS | WINED3D_VIDMEM_ACCOUNTING \
+        | WINED3D_RESTORE_MODE_ON_ACTIVATE | WINED3D_FOCUS_MESSAGES | WINED3D_PIXEL_CENTER_INTEGER)
+
 enum ddraw_device_state
 {
     DDRAW_DEVICE_STATE_OK,
@@ -183,7 +186,6 @@ struct ddraw_surface
     DDSURFACEDESC2          surface_desc;
 
     /* Misc things */
-    DWORD                   uniqueness_value;
     UINT                    mipmap_level;
 
     /* Clipper objects */
