@@ -1,7 +1,7 @@
 /*
  *    Text layout/format tests
  *
- * Copyright 2012 Nikolay Sivov for CodeWeavers
+ * Copyright 2012, 2014-2015 Nikolay Sivov for CodeWeavers
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,8 +28,6 @@
 
 #include "wine/test.h"
 
-static IDWriteFactory *factory;
-
 static const WCHAR tahomaW[] = {'T','a','h','o','m','a',0};
 static const WCHAR enusW[] = {'e','n','-','u','s',0};
 
@@ -41,6 +39,14 @@ static void _expect_ref(IUnknown* obj, ULONG ref, int line)
     ok_(__FILE__,line)(rc-1 == ref, "expected refcount %d, got %d\n", ref, rc-1);
 }
 
+static IDWriteFactory *create_factory(void)
+{
+    IDWriteFactory *factory;
+    HRESULT hr = DWriteCreateFactory(DWRITE_FACTORY_TYPE_ISOLATED, &IID_IDWriteFactory, (IUnknown**)&factory);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    return factory;
+}
+
 enum drawcall_kind {
     DRAW_GLYPHRUN = 0,
     DRAW_UNDERLINE,
@@ -309,9 +315,14 @@ static IDWriteTextRenderer testrenderer = { &testrenderervtbl };
 static void test_CreateTextLayout(void)
 {
     static const WCHAR strW[] = {'s','t','r','i','n','g',0};
+    IDWriteTextLayout2 *layout2;
     IDWriteTextLayout *layout;
+    IDWriteTextFormat *format;
+    IDWriteFactory *factory;
     HRESULT hr;
 
+    factory = create_factory();
+
     hr = IDWriteFactory_CreateTextLayout(factory, NULL, 0, NULL, 0.0, 0.0, &layout);
     ok(hr == E_INVALIDARG, "got 0x%08x\n", hr);
 
@@ -326,6 +337,63 @@ static void test_CreateTextLayout(void)
 
     hr = IDWriteFactory_CreateTextLayout(factory, strW, 6, NULL, 1000.0, 1000.0, &layout);
     ok(hr == E_INVALIDARG, "got 0x%08x\n", hr);
+
+    hr = IDWriteFactory_CreateTextFormat(factory, tahomaW, NULL, DWRITE_FONT_WEIGHT_NORMAL, DWRITE_FONT_STYLE_NORMAL,
+        DWRITE_FONT_STRETCH_NORMAL, 10.0, enusW, &format);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    EXPECT_REF(format, 1);
+    hr = IDWriteFactory_CreateTextLayout(factory, strW, 6, format, 1000.0, 1000.0, &layout);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    EXPECT_REF(format, 1);
+
+    hr = IDWriteTextLayout_QueryInterface(layout, &IID_IDWriteTextLayout2, (void**)&layout2);
+    if (hr == S_OK) {
+        IDWriteTextLayout1 *layout1;
+        IDWriteTextFormat1 *format1;
+        IDWriteTextFormat *format;
+
+        hr = IDWriteTextLayout2_QueryInterface(layout2, &IID_IDWriteTextLayout1, (void**)&layout1);
+        ok(hr == S_OK, "got 0x%08x\n", hr);
+        IDWriteTextLayout1_Release(layout1);
+
+        EXPECT_REF(layout2, 2);
+        hr = IDWriteTextLayout2_QueryInterface(layout2, &IID_IDWriteTextFormat1, (void**)&format1);
+        ok(hr == S_OK, "got 0x%08x\n", hr);
+        EXPECT_REF(layout2, 3);
+
+        hr = IDWriteTextLayout2_QueryInterface(layout2, &IID_IDWriteTextFormat, (void**)&format);
+        ok(hr == S_OK, "got 0x%08x\n", hr);
+        ok(format == (IDWriteTextFormat*)format1, "got %p, %p\n", format, format1);
+        ok(format != (IDWriteTextFormat*)layout2, "got %p, %p\n", format, layout2);
+        EXPECT_REF(layout2, 4);
+
+        hr = IDWriteTextFormat_QueryInterface(format, &IID_IDWriteTextLayout1, (void**)&layout1);
+        ok(hr == S_OK, "got 0x%08x\n", hr);
+        IDWriteTextLayout1_Release(layout1);
+
+        IDWriteTextFormat1_Release(format1);
+        IDWriteTextFormat_Release(format);
+
+        hr = IDWriteTextLayout_QueryInterface(layout, &IID_IDWriteTextFormat1, (void**)&format1);
+        ok(hr == S_OK, "got 0x%08x\n", hr);
+        EXPECT_REF(layout2, 3);
+
+        hr = IDWriteTextLayout_QueryInterface(layout, &IID_IDWriteTextFormat, (void**)&format);
+        ok(hr == S_OK, "got 0x%08x\n", hr);
+        ok(format == (IDWriteTextFormat*)format1, "got %p, %p\n", format, format1);
+        EXPECT_REF(layout2, 4);
+
+        IDWriteTextFormat1_Release(format1);
+        IDWriteTextFormat_Release(format);
+        IDWriteTextLayout2_Release(layout2);
+    }
+    else
+        win_skip("IDWriteTextLayout2 is not supported.\n");
+
+    IDWriteTextLayout_Release(layout);
+    IDWriteTextFormat_Release(format);
+    IDWriteFactory_Release(factory);
 }
 
 static void test_CreateGdiCompatibleTextLayout(void)
@@ -333,9 +401,12 @@ static void test_CreateGdiCompatibleTextLayout(void)
     static const WCHAR strW[] = {'s','t','r','i','n','g',0};
     IDWriteTextLayout *layout;
     IDWriteTextFormat *format;
+    IDWriteFactory *factory;
     FLOAT dimension;
     HRESULT hr;
 
+    factory = create_factory();
+
     hr = IDWriteFactory_CreateGdiCompatibleTextLayout(factory, NULL, 0, NULL, 0.0, 0.0, 0.0, NULL, FALSE, &layout);
     ok(hr == E_INVALIDARG, "got 0x%08x\n", hr);
 
@@ -380,6 +451,7 @@ static void test_CreateGdiCompatibleTextLayout(void)
 
     IDWriteTextLayout_Release(layout);
     IDWriteTextFormat_Release(format);
+    IDWriteFactory_Release(factory);
 }
 
 static void test_CreateTextFormat(void)
@@ -395,8 +467,11 @@ static void test_CreateTextFormat(void)
     IDWriteTextFormat *format;
     FLOAT spacing, baseline;
     IDWriteInlineObject *trimmingsign;
+    IDWriteFactory *factory;
     HRESULT hr;
 
+    factory = create_factory();
+
     hr = IDWriteFactory_CreateTextFormat(factory, tahomaW, NULL, DWRITE_FONT_WEIGHT_NORMAL, DWRITE_FONT_STYLE_NORMAL,
         DWRITE_FONT_STRETCH_NORMAL, 10.0, enusW, &format);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -471,6 +546,7 @@ if (0) /* crashes on native */
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
     IDWriteTextFormat_Release(format);
+    IDWriteFactory_Release(factory);
 }
 
 static void test_GetLocaleName(void)
@@ -479,10 +555,13 @@ static void test_GetLocaleName(void)
     static const WCHAR ruW[] = {'r','u',0};
     IDWriteTextLayout *layout;
     IDWriteTextFormat *format, *format2;
+    IDWriteFactory *factory;
     WCHAR buff[10];
     UINT32 len;
     HRESULT hr;
 
+    factory = create_factory();
+
     hr = IDWriteFactory_CreateTextFormat(factory, tahomaW, NULL, DWRITE_FONT_WEIGHT_NORMAL, DWRITE_FONT_STYLE_NORMAL,
         DWRITE_FONT_STRETCH_NORMAL, 10.0, ruW, &format);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -511,6 +590,7 @@ static void test_GetLocaleName(void)
     IDWriteTextLayout_Release(layout);
     IDWriteTextFormat_Release(format);
     IDWriteTextFormat_Release(format2);
+    IDWriteFactory_Release(factory);
 }
 
 static void test_CreateEllipsisTrimmingSign(void)
@@ -518,8 +598,11 @@ static void test_CreateEllipsisTrimmingSign(void)
     DWRITE_BREAK_CONDITION before, after;
     IDWriteTextFormat *format;
     IDWriteInlineObject *sign;
+    IDWriteFactory *factory;
     HRESULT hr;
 
+    factory = create_factory();
+
     hr = IDWriteFactory_CreateTextFormat(factory, tahomaW, NULL, DWRITE_FONT_WEIGHT_NORMAL, DWRITE_FONT_STYLE_NORMAL,
         DWRITE_FONT_STRETCH_NORMAL, 10.0, enusW, &format);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -540,6 +623,7 @@ if (0) /* crashes on native */
 
     IDWriteInlineObject_Release(sign);
     IDWriteTextFormat_Release(format);
+    IDWriteFactory_Release(factory);
 }
 
 static void test_fontweight(void)
@@ -550,9 +634,12 @@ static void test_fontweight(void)
     IDWriteTextLayout *layout;
     DWRITE_FONT_WEIGHT weight;
     DWRITE_TEXT_RANGE range;
+    IDWriteFactory *factory;
     FLOAT size;
     HRESULT hr;
 
+    factory = create_factory();
+
     hr = IDWriteFactory_CreateTextFormat(factory, tahomaW, NULL, DWRITE_FONT_WEIGHT_BOLD, DWRITE_FONT_STYLE_NORMAL,
         DWRITE_FONT_STRETCH_NORMAL, 10.0, ruW, &format);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -627,6 +714,7 @@ static void test_fontweight(void)
     IDWriteTextLayout_Release(layout);
     IDWriteTextFormat_Release(fmt2);
     IDWriteTextFormat_Release(format);
+    IDWriteFactory_Release(factory);
 }
 
 static void test_SetInlineObject(void)
@@ -638,8 +726,11 @@ static void test_SetInlineObject(void)
     IDWriteTextFormat *format;
     IDWriteTextLayout *layout;
     DWRITE_TEXT_RANGE range, r2;
+    IDWriteFactory *factory;
     HRESULT hr;
 
+    factory = create_factory();
+
     hr = IDWriteFactory_CreateTextFormat(factory, tahomaW, NULL, DWRITE_FONT_WEIGHT_BOLD, DWRITE_FONT_STYLE_NORMAL,
         DWRITE_FONT_STRETCH_NORMAL, 10.0, ruW, &format);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -653,6 +744,9 @@ static void test_SetInlineObject(void)
     hr = IDWriteFactory_CreateEllipsisTrimmingSign(factory, format, &inlineobj2);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
+    EXPECT_REF(inlineobj, 1);
+    EXPECT_REF(inlineobj2, 1);
+
     inlinetest = (void*)0x1;
     hr = IDWriteTextLayout_GetInlineObject(layout, 0, &inlinetest, NULL);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -663,6 +757,8 @@ static void test_SetInlineObject(void)
     hr = IDWriteTextLayout_SetInlineObject(layout, inlineobj, range);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
+    EXPECT_REF(inlineobj, 2);
+
     inlinetest = (void*)0x1;
     hr = IDWriteTextLayout_GetInlineObject(layout, 2, &inlinetest, NULL);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -676,6 +772,8 @@ static void test_SetInlineObject(void)
     ok(r2.startPosition == 0 && r2.length == 2, "got %d, %d\n", r2.startPosition, r2.length);
     IDWriteInlineObject_Release(inlinetest);
 
+    EXPECT_REF(inlineobj, 2);
+
     /* get from somewhere inside a range */
     inlinetest = NULL;
     r2.startPosition = r2.length = 100;
@@ -685,6 +783,8 @@ static void test_SetInlineObject(void)
     ok(r2.startPosition == 0 && r2.length == 2, "got %d, %d\n", r2.startPosition, r2.length);
     IDWriteInlineObject_Release(inlinetest);
 
+    EXPECT_REF(inlineobj, 2);
+
     range.startPosition = 1;
     range.length = 1;
     hr = IDWriteTextLayout_SetInlineObject(layout, inlineobj2, range);
@@ -698,6 +798,9 @@ static void test_SetInlineObject(void)
     ok(r2.startPosition == 1 && r2.length == 1, "got %d, %d\n", r2.startPosition, r2.length);
     IDWriteInlineObject_Release(inlinetest);
 
+    EXPECT_REF(inlineobj, 2);
+    EXPECT_REF(inlineobj2, 2);
+
     inlinetest = NULL;
     r2.startPosition = r2.length = 100;
     hr = IDWriteTextLayout_GetInlineObject(layout, 0, &inlinetest, &r2);
@@ -706,6 +809,8 @@ static void test_SetInlineObject(void)
     ok(r2.startPosition == 0 && r2.length == 1, "got %d, %d\n", r2.startPosition, r2.length);
     IDWriteInlineObject_Release(inlinetest);
 
+    EXPECT_REF(inlineobj, 2);
+
     range.startPosition = 1;
     range.length = 1;
     hr = IDWriteTextLayout_SetInlineObject(layout, inlineobj, range);
@@ -718,11 +823,15 @@ static void test_SetInlineObject(void)
     ok(r2.startPosition == 0 && r2.length == 2, "got %d, %d\n", r2.startPosition, r2.length);
     IDWriteInlineObject_Release(inlinetest);
 
+    EXPECT_REF(inlineobj, 2);
+
     range.startPosition = 1;
     range.length = 2;
     hr = IDWriteTextLayout_SetInlineObject(layout, inlineobj, range);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
+    EXPECT_REF(inlineobj, 2);
+
     r2.startPosition = r2.length = 100;
     hr = IDWriteTextLayout_GetInlineObject(layout, 0, &inlinetest, &r2);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -730,8 +839,17 @@ static void test_SetInlineObject(void)
     ok(r2.startPosition == 0 && r2.length == 3, "got %d, %d\n", r2.startPosition, r2.length);
     IDWriteInlineObject_Release(inlinetest);
 
+    EXPECT_REF(inlineobj, 2);
+    EXPECT_REF(inlineobj2, 1);
+
     IDWriteTextLayout_Release(layout);
+
+    EXPECT_REF(inlineobj, 1);
+
+    IDWriteInlineObject_Release(inlineobj);
+    IDWriteInlineObject_Release(inlineobj2);
     IDWriteTextFormat_Release(format);
+    IDWriteFactory_Release(factory);
 }
 
 /* drawing calls sequence doesn't depend on run order, instead all runs are
@@ -747,7 +865,17 @@ static const struct drawcall_entry draw_seq[] = {
     { DRAW_LAST_KIND }
 };
 
-static void test_draw_sequence(void)
+static const struct drawcall_entry draw_seq2[] = {
+    { DRAW_GLYPHRUN },
+    { DRAW_GLYPHRUN },
+    { DRAW_GLYPHRUN },
+    { DRAW_GLYPHRUN },
+    { DRAW_GLYPHRUN },
+    { DRAW_GLYPHRUN },
+    { DRAW_LAST_KIND }
+};
+
+static void test_Draw(void)
 {
     static const WCHAR strW[] = {'s','t','r','i','n','g',0};
     static const WCHAR ruW[] = {'r','u',0};
@@ -756,8 +884,11 @@ static void test_draw_sequence(void)
     IDWriteTextFormat *format;
     IDWriteTextLayout *layout;
     DWRITE_TEXT_RANGE range;
+    IDWriteFactory *factory;
     HRESULT hr;
 
+    factory = create_factory();
+
     hr = IDWriteFactory_CreateTextFormat(factory, tahomaW, NULL, DWRITE_FONT_WEIGHT_BOLD, DWRITE_FONT_STYLE_NORMAL,
         DWRITE_FONT_STRETCH_NORMAL, 10.0, ruW, &format);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -793,18 +924,32 @@ static void test_draw_sequence(void)
 todo_wine
     ok(hr == S_OK, "got 0x%08x\n", hr);
     ok_sequence(sequences, RENDERER_ID, draw_seq, "draw test", TRUE);
-
-    IDWriteTextFormat_Release(format);
     IDWriteTextLayout_Release(layout);
+
+    /* with reduced width DrawGlyphRun() is called for every line */
+    hr = IDWriteFactory_CreateTextLayout(factory, strW, 6, format, 5.0, 100.0, &layout);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    flush_sequence(sequences, RENDERER_ID);
+    hr = IDWriteTextLayout_Draw(layout, NULL, &testrenderer, 0.0, 0.0);
+todo_wine
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok_sequence(sequences, RENDERER_ID, draw_seq2, "draw test 2", TRUE);
+
+    IDWriteTextLayout_Release(layout);
+    IDWriteTextFormat_Release(format);
+    IDWriteFactory_Release(factory);
 }
 
 static void test_typography(void)
 {
     DWRITE_FONT_FEATURE feature;
     IDWriteTypography *typography;
+    IDWriteFactory *factory;
     UINT32 count;
     HRESULT hr;
 
+    factory = create_factory();
+
     hr = IDWriteFactory_CreateTypography(factory, &typography);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
@@ -841,16 +986,360 @@ static void test_typography(void)
     ok(hr == E_INVALIDARG, "got 0x%08x\n", hr);
 
     IDWriteTypography_Release(typography);
+    IDWriteFactory_Release(factory);
+}
+
+static void test_GetClusterMetrics(void)
+{
+    static const WCHAR strW[] = {'a','b','c','d',0};
+    DWRITE_INLINE_OBJECT_METRICS inline_metrics;
+    DWRITE_CLUSTER_METRICS metrics[4];
+    IDWriteInlineObject *trimm;
+    IDWriteTextFormat *format;
+    IDWriteTextLayout *layout;
+    DWRITE_TEXT_RANGE range;
+    IDWriteFactory *factory;
+    UINT32 count, i;
+    HRESULT hr;
+
+    factory = create_factory();
+
+    hr = IDWriteFactory_CreateTextFormat(factory, tahomaW, NULL, DWRITE_FONT_WEIGHT_NORMAL, DWRITE_FONT_STYLE_NORMAL,
+        DWRITE_FONT_STRETCH_NORMAL, 10.0, enusW, &format);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IDWriteFactory_CreateTextLayout(factory, strW, 4, format, 1000.0, 1000.0, &layout);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    count = 0;
+    hr = IDWriteTextLayout_GetClusterMetrics(layout, NULL, 0, &count);
+    ok(hr == E_NOT_SUFFICIENT_BUFFER, "got 0x%08x\n", hr);
+    ok(count == 4, "got %u\n", count);
+
+    /* check every cluster width */
+    count = 0;
+    hr = IDWriteTextLayout_GetClusterMetrics(layout, metrics, sizeof(metrics)/sizeof(metrics[0]), &count);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(count == 4, "got %u\n", count);
+    for (i = 0; i < count; i++) {
+        ok(metrics[i].width > 0.0, "%u: got width %.2f\n", i, metrics[i].width);
+        ok(metrics[i].length == 1, "%u: got length %u\n", i, metrics[i].length);
+    }
+
+    hr = IDWriteFactory_CreateEllipsisTrimmingSign(factory, format, &trimm);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    range.startPosition = 0;
+    range.length = 2;
+    hr = IDWriteTextLayout_SetInlineObject(layout, trimm, range);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    /* inline object takes a separate cluster, replaced codepoints number doesn't matter */
+    count = 0;
+    hr = IDWriteTextLayout_GetClusterMetrics(layout, NULL, 0, &count);
+    ok(hr == E_NOT_SUFFICIENT_BUFFER, "got 0x%08x\n", hr);
+    ok(count == 3, "got %u\n", count);
+
+    count = 0;
+    memset(&metrics, 0, sizeof(metrics));
+    hr = IDWriteTextLayout_GetClusterMetrics(layout, metrics, 1, &count);
+    ok(hr == E_NOT_SUFFICIENT_BUFFER, "got 0x%08x\n", hr);
+    ok(count == 3, "got %u\n", count);
+    ok(metrics[0].length == 2, "got %u\n", metrics[0].length);
+
+    hr = IDWriteInlineObject_GetMetrics(trimm, &inline_metrics);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+todo_wine
+    ok(inline_metrics.width > 0.0 && inline_metrics.width == metrics[0].width, "got %.2f, expected %.2f\n",
+        inline_metrics.width, metrics[0].width);
+
+    IDWriteInlineObject_Release(trimm);
+    IDWriteTextLayout_Release(layout);
+    IDWriteTextFormat_Release(format);
+    IDWriteFactory_Release(factory);
+}
+
+static void test_SetLocaleName(void)
+{
+    static const WCHAR strW[] = {'a','b','c','d',0};
+    WCHAR buffW[LOCALE_NAME_MAX_LENGTH+sizeof(strW)/sizeof(WCHAR)];
+    IDWriteTextFormat *format;
+    IDWriteTextLayout *layout;
+    DWRITE_TEXT_RANGE range;
+    IDWriteFactory *factory;
+    HRESULT hr;
+
+    factory = create_factory();
+
+    hr = IDWriteFactory_CreateTextFormat(factory, tahomaW, NULL, DWRITE_FONT_WEIGHT_NORMAL, DWRITE_FONT_STYLE_NORMAL,
+        DWRITE_FONT_STRETCH_NORMAL, 10.0, enusW, &format);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IDWriteFactory_CreateTextLayout(factory, strW, 4, format, 1000.0, 1000.0, &layout);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    range.startPosition = 0;
+    range.length = 1;
+    hr = IDWriteTextLayout_SetLocaleName(layout, enusW, range);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IDWriteTextLayout_SetLocaleName(layout, NULL, range);
+    ok(hr == E_INVALIDARG, "got 0x%08x\n", hr);
+
+    /* invalid locale name is allowed */
+    hr = IDWriteTextLayout_SetLocaleName(layout, strW, range);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IDWriteTextLayout_GetLocaleName(layout, 0, NULL, 0, NULL);
+    ok(hr == E_INVALIDARG, "got 0x%08x\n", hr);
+
+if (0) /* crashes on native */
+    hr = IDWriteTextLayout_GetLocaleName(layout, 0, NULL, 1, NULL);
+
+    buffW[0] = 0;
+    hr = IDWriteTextLayout_GetLocaleName(layout, 0, buffW, sizeof(buffW)/sizeof(WCHAR), NULL);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(!lstrcmpW(buffW, strW), "got %s\n", wine_dbgstr_w(buffW));
+
+    /* get with a shorter buffer */
+    buffW[0] = 0xa;
+    hr = IDWriteTextLayout_GetLocaleName(layout, 0, buffW, 1, NULL);
+    ok(hr == E_NOT_SUFFICIENT_BUFFER, "got 0x%08x\n", hr);
+    ok(buffW[0] == 0, "got %x\n", buffW[0]);
+
+    /* name is too long */
+    lstrcpyW(buffW, strW);
+    while (lstrlenW(buffW) <= LOCALE_NAME_MAX_LENGTH)
+        lstrcatW(buffW, strW);
+
+    range.startPosition = 0;
+    range.length = 1;
+    hr = IDWriteTextLayout_SetLocaleName(layout, buffW, range);
+    ok(hr == E_INVALIDARG, "got 0x%08x\n", hr);
+
+    buffW[0] = 0;
+    hr = IDWriteTextLayout_GetLocaleName(layout, 0, buffW, sizeof(buffW)/sizeof(WCHAR), NULL);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(!lstrcmpW(buffW, strW), "got %s\n", wine_dbgstr_w(buffW));
+
+    IDWriteTextLayout_Release(layout);
+    IDWriteTextFormat_Release(format);
+    IDWriteFactory_Release(factory);
+}
+
+static void test_SetPairKerning(void)
+{
+    static const WCHAR strW[] = {'a','b','c','d',0};
+    IDWriteTextLayout1 *layout1;
+    IDWriteTextFormat *format;
+    IDWriteTextLayout *layout;
+    DWRITE_TEXT_RANGE range;
+    IDWriteFactory *factory;
+    BOOL kerning;
+    HRESULT hr;
+
+    factory = create_factory();
+
+    hr = IDWriteFactory_CreateTextFormat(factory, tahomaW, NULL, DWRITE_FONT_WEIGHT_NORMAL, DWRITE_FONT_STYLE_NORMAL,
+        DWRITE_FONT_STRETCH_NORMAL, 10.0, enusW, &format);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IDWriteFactory_CreateTextLayout(factory, strW, 4, format, 1000.0, 1000.0, &layout);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    IDWriteTextFormat_Release(format);
+
+    hr = IDWriteTextLayout_QueryInterface(layout, &IID_IDWriteTextLayout1, (void**)&layout1);
+    IDWriteTextLayout_Release(layout);
+
+    if (hr != S_OK) {
+        win_skip("SetPairKerning() is not supported.\n");
+        IDWriteFactory_Release(factory);
+        return;
+    }
+
+if (0) { /* crashes on native */
+    hr = IDWriteTextLayout1_GetPairKerning(layout1, 0, NULL, NULL);
+    hr = IDWriteTextLayout1_GetPairKerning(layout1, 0, NULL, &range);
+}
+
+    hr = IDWriteTextLayout1_GetPairKerning(layout1, 0, &kerning, NULL);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    range.startPosition = 0;
+    range.length = 0;
+    kerning = TRUE;
+    hr = IDWriteTextLayout1_GetPairKerning(layout1, 0, &kerning, &range);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(!kerning, "got %d\n", kerning);
+
+    range.startPosition = 0;
+    range.length = 1;
+    hr = IDWriteTextLayout1_SetPairKerning(layout1, 2, range);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    kerning = FALSE;
+    hr = IDWriteTextLayout1_GetPairKerning(layout1, 0, &kerning, &range);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(kerning == TRUE, "got %d\n", kerning);
+
+    IDWriteTextLayout1_Release(layout1);
+    IDWriteFactory_Release(factory);
+}
+
+static void test_SetVerticalGlyphOrientation(void)
+{
+    static const WCHAR strW[] = {'a','b','c','d',0};
+    DWRITE_VERTICAL_GLYPH_ORIENTATION orientation;
+    IDWriteTextLayout2 *layout2;
+    IDWriteTextFormat *format;
+    IDWriteTextLayout *layout;
+    IDWriteFactory *factory;
+    HRESULT hr;
+
+    factory = create_factory();
+
+    hr = IDWriteFactory_CreateTextFormat(factory, tahomaW, NULL, DWRITE_FONT_WEIGHT_NORMAL, DWRITE_FONT_STYLE_NORMAL,
+        DWRITE_FONT_STRETCH_NORMAL, 10.0, enusW, &format);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IDWriteFactory_CreateTextLayout(factory, strW, 4, format, 1000.0, 1000.0, &layout);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    IDWriteTextFormat_Release(format);
+
+    hr = IDWriteTextLayout_QueryInterface(layout, &IID_IDWriteTextLayout2, (void**)&layout2);
+    IDWriteTextLayout_Release(layout);
+
+    if (hr != S_OK) {
+        win_skip("SetVerticalGlyphOrientation() is not supported.\n");
+        IDWriteFactory_Release(factory);
+        return;
+    }
+
+    orientation = IDWriteTextLayout2_GetVerticalGlyphOrientation(layout2);
+    ok(orientation == DWRITE_VERTICAL_GLYPH_ORIENTATION_DEFAULT, "got %d\n", orientation);
+
+    hr = IDWriteTextLayout2_SetVerticalGlyphOrientation(layout2, DWRITE_VERTICAL_GLYPH_ORIENTATION_STACKED+1);
+    ok(hr == E_INVALIDARG, "got 0x%08x\n", hr);
+
+    IDWriteTextLayout2_Release(layout2);
+    IDWriteFactory_Release(factory);
+}
+
+static void test_fallback(void)
+{
+    static const WCHAR strW[] = {'a','b','c','d',0};
+    IDWriteFontFallback *fallback, *fallback2;
+    IDWriteTextLayout2 *layout2;
+    IDWriteTextFormat1 *format1;
+    IDWriteTextFormat *format;
+    IDWriteTextLayout *layout;
+    IDWriteFactory2 *factory2;
+    IDWriteFactory *factory;
+    HRESULT hr;
+
+    factory = create_factory();
+
+    hr = IDWriteFactory_CreateTextFormat(factory, tahomaW, NULL, DWRITE_FONT_WEIGHT_NORMAL, DWRITE_FONT_STYLE_NORMAL,
+        DWRITE_FONT_STRETCH_NORMAL, 10.0, enusW, &format);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IDWriteFactory_CreateTextLayout(factory, strW, 4, format, 1000.0, 1000.0, &layout);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    IDWriteTextFormat_Release(format);
+
+    hr = IDWriteTextLayout_QueryInterface(layout, &IID_IDWriteTextLayout2, (void**)&layout2);
+    IDWriteTextLayout_Release(layout);
+
+    if (hr != S_OK) {
+        win_skip("GetFontFallback() is not supported.\n");
+        IDWriteFactory_Release(factory);
+        return;
+    }
+
+if (0) /* crashes on native */
+    hr = IDWriteTextLayout2_GetFontFallback(layout2, NULL);
+
+    fallback = (void*)0xdeadbeef;
+    hr = IDWriteTextLayout2_GetFontFallback(layout2, &fallback);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(fallback == NULL, "got %p\n", fallback);
+
+    hr = IDWriteTextLayout2_QueryInterface(layout2, &IID_IDWriteTextFormat1, (void**)&format1);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    fallback = (void*)0xdeadbeef;
+    hr = IDWriteTextFormat1_GetFontFallback(format1, &fallback);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(fallback == NULL, "got %p\n", fallback);
+
+    hr = IDWriteFactory_QueryInterface(factory, &IID_IDWriteFactory2, (void**)&factory2);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    fallback = NULL;
+    hr = IDWriteFactory2_GetSystemFontFallback(factory2, &fallback);
+todo_wine
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+if (hr == S_OK) {
+    ok(fallback != NULL, "got %p\n", fallback);
+
+    hr = IDWriteTextFormat1_SetFontFallback(format1, fallback);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    fallback2 = (void*)0xdeadbeef;
+    hr = IDWriteTextLayout2_GetFontFallback(layout2, &fallback2);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(fallback2 == fallback, "got %p\n", fallback2);
+
+    hr = IDWriteTextLayout2_SetFontFallback(layout2, NULL);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    fallback2 = (void*)0xdeadbeef;
+    hr = IDWriteTextFormat1_GetFontFallback(format1, &fallback2);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(fallback2 == NULL, "got %p\n", fallback2);
+
+    IDWriteFontFallback_Release(fallback);
+}
+    IDWriteTextFormat1_Release(format1);
+    IDWriteTextLayout2_Release(layout2);
+    IDWriteFactory_Release(factory);
+}
+
+static void test_DetermineMinWidth(void)
+{
+    static const WCHAR strW[] = {'a','b','c','d',0};
+    IDWriteTextFormat *format;
+    IDWriteTextLayout *layout;
+    IDWriteFactory *factory;
+    FLOAT minwidth;
+    HRESULT hr;
+
+    factory = create_factory();
+
+    hr = IDWriteFactory_CreateTextFormat(factory, tahomaW, NULL, DWRITE_FONT_WEIGHT_NORMAL, DWRITE_FONT_STYLE_NORMAL,
+        DWRITE_FONT_STRETCH_NORMAL, 10.0, enusW, &format);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IDWriteFactory_CreateTextLayout(factory, strW, lstrlenW(strW), format, 1000.0, 1000.0, &layout);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IDWriteTextLayout_DetermineMinWidth(layout, NULL);
+    ok(hr == E_INVALIDARG, "got 0x%08x\n", hr);
+
+    minwidth = 0.0;
+    hr = IDWriteTextLayout_DetermineMinWidth(layout, &minwidth);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(minwidth > 0.0, "got %.2f\n", minwidth);
+
+    IDWriteTextLayout_Release(layout);
+    IDWriteTextFormat_Release(format);
 }
 
 START_TEST(layout)
 {
-    HRESULT hr;
+    IDWriteFactory *factory;
 
-    hr = DWriteCreateFactory(DWRITE_FACTORY_TYPE_ISOLATED, &IID_IDWriteFactory, (IUnknown**)&factory);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-    if (hr != S_OK)
-    {
+    if (!(factory = create_factory())) {
         win_skip("failed to create factory\n");
         return;
     }
@@ -865,8 +1354,14 @@ START_TEST(layout)
     test_CreateEllipsisTrimmingSign();
     test_fontweight();
     test_SetInlineObject();
-    test_draw_sequence();
+    test_Draw();
     test_typography();
+    test_GetClusterMetrics();
+    test_SetLocaleName();
+    test_SetPairKerning();
+    test_SetVerticalGlyphOrientation();
+    test_fallback();
+    test_DetermineMinWidth();
 
     IDWriteFactory_Release(factory);
 }
