@@ -65,7 +65,7 @@ static HRESULT navigate_anchor_window(HTMLAnchorElement *This, const WCHAR *targ
     if(FAILED(hres))
         return hres;
 
-    hres = navigate_new_window(This->element.node.doc->basedoc.window, uri, target, NULL);
+    hres = navigate_new_window(This->element.node.doc->basedoc.window, uri, target, NULL, NULL);
     IUri_Release(uri);
     return hres;
 }
@@ -724,7 +724,7 @@ static HRESULT HTMLAnchorElement_handle_event(HTMLDOMNode *iface, eventid_t eid,
 
     if(eid == EVENTID_CLICK) {
         nsIDOMMouseEvent *mouse_event;
-        UINT16 button;
+        INT16 button;
         nsresult nsres;
 
         TRACE("CLICK\n");
