@@ -95,7 +95,8 @@ static inline HRESULT set_window_event(HTMLWindow *window, eventid_t eid, VARIAN
         return E_FAIL;
     }
 
-    return set_event_handler(&window->inner_window->doc->body_event_target, window->inner_window->doc, eid, var);
+    return set_event_handler(&window->inner_window->dispex, &window->inner_window->doc->body_event_target,
+                             window->inner_window->doc, eid, var);
 }
 
 static inline HRESULT get_window_event(HTMLWindow *window, eventid_t eid, VARIANT *var)
@@ -105,7 +106,7 @@ static inline HRESULT get_window_event(HTMLWindow *window, eventid_t eid, VARIAN
         return E_FAIL;
     }
 
-    return get_event_handler(&window->inner_window->doc->body_event_target, eid, var);
+    return get_event_handler(&window->inner_window->dispex, &window->inner_window->doc->body_event_target, eid, var);
 }
 
 static void detach_inner_window(HTMLInnerWindow *window)
@@ -932,6 +933,8 @@ static HRESULT WINAPI HTMLWindow2_open(IHTMLWindow2 *iface, BSTR url, BSTR name,
     IUri *uri;
     HRESULT hres;
 
+    static const WCHAR _selfW[] = {'_','s','e','l','f',0};
+
     TRACE("(%p)->(%s %s %s %x %p)\n", This, debugstr_w(url), debugstr_w(name),
           debugstr_w(features), replace, pomWindowResult);
 
@@ -939,6 +942,23 @@ static HRESULT WINAPI HTMLWindow2_open(IHTMLWindow2 *iface, BSTR url, BSTR name,
         return E_UNEXPECTED;
 
     if(name && *name == '_') {
+        if(!strcmpW(name, _selfW)) {
+            if((features && *features) || replace)
+                FIXME("Unsupported arguments for _self target\n");
+
+            hres = IHTMLWindow2_navigate(&This->IHTMLWindow2_iface, url);
+            if(FAILED(hres))
+                return hres;
+
+            if(pomWindowResult) {
+                FIXME("Returning this window for _self target\n");
+                *pomWindowResult = &This->IHTMLWindow2_iface;
+                IHTMLWindow2_AddRef(*pomWindowResult);
+            }
+
+            return S_OK;
+        }
+
         FIXME("Unsupported name %s\n", debugstr_w(name));
         return E_NOTIMPL;
     }
@@ -967,7 +987,7 @@ static HRESULT WINAPI HTMLWindow2_open(IHTMLWindow2 *iface, BSTR url, BSTR name,
     if(FAILED(hres))
         return hres;
 
-    hres = navigate_new_window(window, uri, name, pomWindowResult);
+    hres = navigate_new_window(window, uri, name, NULL, pomWindowResult);
     IUri_Release(uri);
     return hres;
 }
@@ -1594,15 +1614,33 @@ static HRESULT WINAPI HTMLWindow3_Invoke(IHTMLWindow3 *iface, DISPID dispIdMembe
 static HRESULT WINAPI HTMLWindow3_get_screenLeft(IHTMLWindow3 *iface, LONG *p)
 {
     HTMLWindow *This = impl_from_IHTMLWindow3(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    nsresult nsres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsres = nsIDOMWindow_GetScreenX(This->outer_window->nswindow, p);
+    if(NS_FAILED(nsres)) {
+        ERR("GetScreenX failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLWindow3_get_screenTop(IHTMLWindow3 *iface, LONG *p)
 {
     HTMLWindow *This = impl_from_IHTMLWindow3(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    nsresult nsres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsres = nsIDOMWindow_GetScreenY(This->outer_window->nswindow, p);
+    if(NS_FAILED(nsres)) {
+        ERR("GetScreenY failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLWindow3_attachEvent(IHTMLWindow3 *iface, BSTR event, IDispatch *pDisp, VARIANT_BOOL *pfResult)
