@@ -702,12 +702,58 @@ static ScriptHost *create_script_host(HTMLInnerWindow *window, const GUID *guid)
     return ret;
 }
 
+typedef struct {
+    task_t header;
+    HTMLScriptElement *elem;
+} fire_readystatechange_task_t;
+
+static void fire_readystatechange_proc(task_t *_task)
+{
+    fire_readystatechange_task_t *task = (fire_readystatechange_task_t*)_task;
+
+    if(!task->elem->pending_readystatechange_event)
+        return;
+
+    task->elem->pending_readystatechange_event = FALSE;
+    fire_event(task->elem->element.node.doc, EVENTID_READYSTATECHANGE, FALSE, task->elem->element.node.nsnode, NULL, NULL);
+}
+
+static void fire_readystatechange_task_destr(task_t *_task)
+{
+    fire_readystatechange_task_t *task = (fire_readystatechange_task_t*)_task;
+
+    IHTMLScriptElement_Release(&task->elem->IHTMLScriptElement_iface);
+}
+
 static void set_script_elem_readystate(HTMLScriptElement *script_elem, READYSTATE readystate)
 {
     script_elem->readystate = readystate;
 
-    if(readystate != READYSTATE_INTERACTIVE)
-        fire_event(script_elem->element.node.doc, EVENTID_READYSTATECHANGE, FALSE, script_elem->element.node.nsnode, NULL, NULL);
+    if(readystate != READYSTATE_INTERACTIVE) {
+        if(!script_elem->element.node.doc->window->parser_callback_cnt) {
+            fire_readystatechange_task_t *task;
+            HRESULT hres;
+
+            if(script_elem->pending_readystatechange_event)
+                return;
+
+            task = heap_alloc(sizeof(*task));
+            if(!task)
+                return;
+
+            IHTMLScriptElement_AddRef(&script_elem->IHTMLScriptElement_iface);
+            task->elem = script_elem;
+
+            hres = push_task(&task->header, fire_readystatechange_proc, fire_readystatechange_task_destr,
+                    script_elem->element.node.doc->window->task_magic);
+            if(SUCCEEDED(hres))
+                script_elem->pending_readystatechange_event = TRUE;
+        }else {
+            script_elem->pending_readystatechange_event = FALSE;
+            fire_event(script_elem->element.node.doc, EVENTID_READYSTATECHANGE, FALSE,
+                    script_elem->element.node.nsnode, NULL, NULL);
+        }
+    }
 }
 
 static void parse_elem_text(ScriptHost *script_host, HTMLScriptElement *script_elem, LPCWSTR text)
@@ -858,6 +904,7 @@ static const BSCallbackVtbl ScriptBSCVtbl = {
 
 static HRESULT bind_script_to_text(HTMLInnerWindow *window, IUri *uri, HTMLScriptElement *script_elem, WCHAR **ret)
 {
+    UINT cp = CP_UTF8;
     ScriptBSC *bsc;
     IMoniker *mon;
     WCHAR *text;
@@ -915,18 +962,21 @@ static HRESULT bind_script_to_text(HTMLInnerWindow *window, IUri *uri, HTMLScrip
         text[bsc->bsc.readed/sizeof(WCHAR)] = 0;
         break;
 
-    case BOM_UTF8:
-    default: {
+    default:
+        /* FIXME: Try to use charset from HTTP headers first */
+        cp = get_document_charset(window->doc);
+        /* fall through */
+    case BOM_UTF8: {
         DWORD len;
 
-        len = MultiByteToWideChar(CP_UTF8, 0, bsc->buf, bsc->bsc.readed, NULL, 0);
+        len = MultiByteToWideChar(cp, 0, bsc->buf, bsc->bsc.readed, NULL, 0);
         text = heap_alloc((len+1)*sizeof(WCHAR));
         if(!text) {
             hres = E_OUTOFMEMORY;
             break;
         }
 
-        MultiByteToWideChar(CP_UTF8, 0, bsc->buf, bsc->bsc.readed, text, len);
+        MultiByteToWideChar(cp, 0, bsc->buf, bsc->bsc.readed, text, len);
         text[len] = 0;
     }
     }
@@ -1416,7 +1466,7 @@ void bind_event_scripts(HTMLDocumentNode *doc)
         return;
 
     nsAString_InitDepend(&selector_str, selectorW);
-    nsres = nsIDOMNodeSelector_QuerySelectorAll(doc->nsnode_selector, &selector_str, &node_list);
+    nsres = nsIDOMHTMLDocument_QuerySelectorAll(doc->nsdoc, &selector_str, &node_list);
     nsAString_Finish(&selector_str);
     if(NS_FAILED(nsres)) {
         ERR("QuerySelectorAll failed: %08x\n", nsres);
@@ -1459,8 +1509,11 @@ void bind_event_scripts(HTMLDocumentNode *doc)
                 else
                     bind_node_event(doc, event_target, target_node, event, event_disp);
 
-                if(target_node)
+                if(target_node) {
                     IHTMLDOMNode_Release(&target_node->IHTMLDOMNode_iface);
+                    if(plugin_container)
+                        node_release(&plugin_container->element.node);
+                }
             }
 
             heap_free(event);
