@@ -32,15 +32,31 @@ WINE_DEFAULT_DEBUG_CHANNEL(msvcp);
 
 void WINAPI _CxxThrowException(exception*,const cxx_exception_type*);
 
+#if _MSVCP_VER >= 70
+typedef const char **exception_name;
+#define EXCEPTION_STR(name) (*name)
+#define EXCEPTION_NAME(str) ((exception_name)&str)
+#else
+typedef const char *exception_name;
+#define EXCEPTION_STR(name) (name)
+#define EXCEPTION_NAME(str) (str)
+#endif
+
 /* vtables */
 extern const vtable_ptr MSVCP_exception_vtable;
+/* ??_7bad_alloc@std@@6B@ */
 extern const vtable_ptr MSVCP_bad_alloc_vtable;
+/* ??_7logic_error@std@@6B@ */
 extern const vtable_ptr MSVCP_logic_error_vtable;
+/* ??_7length_error@std@@6B@ */
 extern const vtable_ptr MSVCP_length_error_vtable;
+/* ??_7out_of_range@std@@6B@ */
 extern const vtable_ptr MSVCP_out_of_range_vtable;
 extern const vtable_ptr MSVCP_invalid_argument_vtable;
+/* ??_7runtime_error@std@@6B@ */
 extern const vtable_ptr MSVCP_runtime_error_vtable;
 extern const vtable_ptr MSVCP_failure_vtable;
+/* ??_7bad_cast@std@@6B@ */
 extern const vtable_ptr MSVCP_bad_cast_vtable;
 
 static void MSVCP_type_info_dtor(type_info * _this)
@@ -71,15 +87,15 @@ void * __thiscall MSVCP_type_info_vector_dtor(type_info * _this, unsigned int fl
 
 DEFINE_RTTI_DATA0( type_info, 0, ".?AVtype_info@@" )
 
-static exception* MSVCP_exception_ctor(exception *this, const char **name)
+static exception* MSVCP_exception_ctor(exception *this, exception_name name)
 {
-    TRACE("(%p %s)\n", this, *name);
+    TRACE("(%p %s)\n", this, EXCEPTION_STR(name));
 
     this->vtable = &MSVCP_exception_vtable;
-    if(*name) {
-        unsigned int name_len = strlen(*name) + 1;
+    if(EXCEPTION_STR(name)) {
+        unsigned int name_len = strlen(EXCEPTION_STR(name)) + 1;
         this->name = malloc(name_len);
-        memcpy(this->name, *name, name_len);
+        memcpy(this->name, EXCEPTION_STR(name), name_len);
         this->do_free = TRUE;
     } else {
         this->name = NULL;
@@ -98,7 +114,7 @@ exception* __thiscall MSVCP_exception_copy_ctor(exception *this, const exception
         this->name = rhs->name;
         this->do_free = FALSE;
     } else
-        MSVCP_exception_ctor(this, (const char**)&rhs->name);
+        MSVCP_exception_ctor(this, EXCEPTION_NAME(rhs->name));
     TRACE("name = %s\n", this->name);
     return this;
 }
@@ -135,17 +151,56 @@ void * __thiscall MSVCP_exception_vector_dtor(exception *this, unsigned int flag
 DEFINE_RTTI_DATA0(exception, 0, ".?AVexception@std@@")
 DEFINE_CXX_DATA0(exception, MSVCP_exception_dtor)
 
+/* ?_Doraise@bad_alloc@std@@MBEXXZ */
+/* ?_Doraise@bad_alloc@std@@MEBAXXZ */
+/* ?_Doraise@logic_error@std@@MBEXXZ */
+/* ?_Doraise@logic_error@std@@MEBAXXZ */
+/* ?_Doraise@length_error@std@@MBEXXZ */
+/* ?_Doraise@length_error@std@@MEBAXXZ */
+/* ?_Doraise@out_of_range@std@@MBEXXZ */
+/* ?_Doraise@out_of_range@std@@MEBAXXZ */
+/* ?_Doraise@runtime_error@std@@MBEXXZ */
+/* ?_Doraise@runtime_error@std@@MEBAXXZ */
+/* ?_Doraise@bad_cast@std@@MBEXXZ */
+/* ?_Doraise@bad_cast@std@@MEBAXXZ */
+DEFINE_THISCALL_WRAPPER(MSVCP_exception__Doraise, 4)
+void __thiscall MSVCP_exception__Doraise(exception *this)
+{
+    FIXME("(%p) stub\n", this);
+}
+
+DEFINE_THISCALL_WRAPPER(MSVCP_exception_what,4)
+const char* __thiscall MSVCP_exception_what(exception * this)
+{
+    TRACE("(%p) returning %s\n", this, this->name);
+    return this->name ? this->name : "Unknown exception";
+}
+
 /* bad_alloc class data */
 typedef exception bad_alloc;
 
-static bad_alloc* MSVCP_bad_alloc_ctor(bad_alloc *this, const char **name)
+/* ??0bad_alloc@std@@QAE@PBD@Z */
+/* ??0bad_alloc@std@@QEAA@PEBD@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_bad_alloc_ctor, 8)
+bad_alloc* __thiscall MSVCP_bad_alloc_ctor(bad_alloc *this, exception_name name)
 {
-    TRACE("%p %s\n", this, *name);
+    TRACE("%p %s\n", this, EXCEPTION_STR(name));
     MSVCP_exception_ctor(this, name);
     this->vtable = &MSVCP_bad_alloc_vtable;
     return this;
 }
 
+/* ??0bad_alloc@std@@QAE@XZ */
+/* ??0bad_alloc@std@@QEAA@XZ */
+DEFINE_THISCALL_WRAPPER(MSVCP_bad_alloc_default_ctor, 4)
+bad_alloc* __thiscall MSVCP_bad_alloc_default_ctor(bad_alloc *this)
+{
+    static const char name[] = "bad allocation";
+    return MSVCP_bad_alloc_ctor(this, EXCEPTION_NAME(name));
+}
+
+/* ??0bad_alloc@std@@QAE@ABV01@@Z */
+/* ??0bad_alloc@std@@QEAA@AEBV01@@Z */
 DEFINE_THISCALL_WRAPPER(MSVCP_bad_alloc_copy_ctor, 8)
 bad_alloc* __thiscall MSVCP_bad_alloc_copy_ctor(bad_alloc *this, const bad_alloc *rhs)
 {
@@ -155,6 +210,8 @@ bad_alloc* __thiscall MSVCP_bad_alloc_copy_ctor(bad_alloc *this, const bad_alloc
     return this;
 }
 
+/* ??1bad_alloc@std@@UAE@XZ */
+/* ??1bad_alloc@std@@UEAA@XZ */
 DEFINE_THISCALL_WRAPPER(MSVCP_bad_alloc_dtor, 4)
 void __thiscall MSVCP_bad_alloc_dtor(bad_alloc *this)
 {
@@ -182,36 +239,39 @@ void * __thiscall MSVCP_bad_alloc_vector_dtor(bad_alloc *this, unsigned int flag
     return this;
 }
 
-DEFINE_THISCALL_WRAPPER(MSVCP_what_exception,4)
-const char* __thiscall MSVCP_what_exception(exception * this)
+/* ??4bad_alloc@std@@QAEAAV01@ABV01@@Z */
+/* ??4bad_alloc@std@@QEAAAEAV01@AEBV01@@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_bad_alloc_assign, 8)
+bad_alloc* __thiscall MSVCP_bad_alloc_assign(bad_alloc *this, const bad_alloc *assign)
 {
-    TRACE("(%p) returning %s\n", this, this->name);
-    return this->name ? this->name : "Unknown exception";
+    MSVCP_bad_alloc_dtor(this);
+    return MSVCP_bad_alloc_copy_ctor(this, assign);
 }
 
 DEFINE_RTTI_DATA1(bad_alloc, 0, &exception_rtti_base_descriptor, ".?AVbad_alloc@std@@")
 DEFINE_CXX_DATA1(bad_alloc, &exception_cxx_type_info, MSVCP_bad_alloc_dtor)
 
 /* logic_error class data */
-typedef struct _logic_error {
+typedef struct {
     exception e;
     basic_string_char str;
 } logic_error;
 
-static logic_error* MSVCP_logic_error_ctor(
-        logic_error *this, const char **name)
+static logic_error* MSVCP_logic_error_ctor( logic_error *this, exception_name name )
 {
-    TRACE("%p %s\n", this, *name);
+    TRACE("%p %s\n", this, EXCEPTION_STR(name));
     this->e.vtable = &MSVCP_logic_error_vtable;
     this->e.name = NULL;
     this->e.do_free = FALSE;
-    MSVCP_basic_string_char_ctor_cstr(&this->str, *name);
+    MSVCP_basic_string_char_ctor_cstr(&this->str, EXCEPTION_STR(name));
     return this;
 }
 
+/* ??0logic_error@std@@QAE@ABV01@@Z */
+/* ??0logic_error@std@@QEAA@AEBV01@@Z */
 DEFINE_THISCALL_WRAPPER(MSVCP_logic_error_copy_ctor, 8)
 logic_error* __thiscall MSVCP_logic_error_copy_ctor(
-        logic_error *this, logic_error *rhs)
+        logic_error *this, const logic_error *rhs)
 {
     TRACE("%p %p\n", this, rhs);
     MSVCP_exception_copy_ctor(&this->e, &rhs->e);
@@ -220,6 +280,22 @@ logic_error* __thiscall MSVCP_logic_error_copy_ctor(
     return this;
 }
 
+/* ??0logic_error@std@@QAE@ABV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@1@@Z */
+/* ??0logic_error@std@@QEAA@AEBV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@1@@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_logic_error_ctor_bstr, 8)
+logic_error* __thiscall MSVCP_logic_error_ctor_bstr(logic_error *this, const basic_string_char *str)
+{
+    const char *name = MSVCP_basic_string_char_c_str(str);
+    TRACE("(%p %p %s)\n", this, str, name);
+    return MSVCP_logic_error_ctor(this, EXCEPTION_NAME(name));
+}
+
+/* ??1logic_error@std@@UAE@XZ */
+/* ??1logic_error@std@@UEAA@XZ */
+/* ??1length_error@std@@UAE@XZ */
+/* ??1length_error@std@@UEAA@XZ */
+/* ??1out_of_range@std@@UAE@XZ */
+/* ??1out_of_range@std@@UEAA@XZ */
 DEFINE_THISCALL_WRAPPER(MSVCP_logic_error_dtor, 4)
 void __thiscall MSVCP_logic_error_dtor(logic_error *this)
 {
@@ -249,6 +325,17 @@ void* __thiscall MSVCP_logic_error_vector_dtor(
     return this;
 }
 
+/* ??4logic_error@std@@QAEAAV01@ABV01@@Z */
+/* ??4logic_error@std@@QEAAAEAV01@AEBV01@@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_logic_error_assign, 8)
+logic_error* __thiscall MSVCP_logic_error_assign(logic_error *this, const logic_error *assign)
+{
+    MSVCP_logic_error_dtor(this);
+    return MSVCP_logic_error_copy_ctor(this, assign);
+}
+
+/* ?what@logic_error@std@@UBEPBDXZ */
+/* ?what@logic_error@std@@UEBAPEBDXZ */
 DEFINE_THISCALL_WRAPPER(MSVCP_logic_error_what, 4)
 const char* __thiscall MSVCP_logic_error_what(logic_error *this)
 {
@@ -262,18 +349,19 @@ DEFINE_CXX_DATA1(logic_error, &exception_cxx_type_info, MSVCP_logic_error_dtor)
 /* length_error class data */
 typedef logic_error length_error;
 
-static length_error* MSVCP_length_error_ctor(
-        length_error *this, const char **name)
+static length_error* MSVCP_length_error_ctor( length_error *this, exception_name name )
 {
-    TRACE("%p %s\n", this, *name);
+    TRACE("%p %s\n", this, EXCEPTION_STR(name));
     MSVCP_logic_error_ctor(this, name);
     this->e.vtable = &MSVCP_length_error_vtable;
     return this;
 }
 
+/* ??0length_error@std@@QAE@ABV01@@Z */
+/* ??0length_error@std@@QEAA@AEBV01@@Z */
 DEFINE_THISCALL_WRAPPER(MSVCP_length_error_copy_ctor, 8)
 length_error* __thiscall MSVCP_length_error_copy_ctor(
-        length_error *this, length_error *rhs)
+        length_error *this, const length_error *rhs)
 {
     TRACE("%p %p\n", this, rhs);
     MSVCP_logic_error_copy_ctor(this, rhs);
@@ -281,24 +369,44 @@ length_error* __thiscall MSVCP_length_error_copy_ctor(
     return this;
 }
 
+/* ??0length_error@std@@QAE@ABV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@1@@Z */
+/* ??0length_error@std@@QEAA@AEBV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@1@@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_length_error_ctor_bstr, 8)
+length_error* __thiscall MSVCP_length_error_ctor_bstr(length_error *this, const basic_string_char *str)
+{
+    const char *name = MSVCP_basic_string_char_c_str(str);
+    TRACE("(%p %p %s)\n", this, str, name);
+    return MSVCP_length_error_ctor(this, EXCEPTION_NAME(name));
+}
+
+/* ??4length_error@std@@QAEAAV01@ABV01@@Z */
+/* ??4length_error@std@@QEAAAEAV01@AEBV01@@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_length_error_assign, 8)
+length_error* __thiscall MSVCP_length_error_assign(length_error *this, const length_error *assign)
+{
+    MSVCP_logic_error_dtor(this);
+    return MSVCP_length_error_copy_ctor(this, assign);
+}
+
 DEFINE_RTTI_DATA2(length_error, 0, &logic_error_rtti_base_descriptor, &exception_rtti_base_descriptor, ".?AVlength_error@std@@")
 DEFINE_CXX_DATA2(length_error, &logic_error_cxx_type_info, &exception_cxx_type_info, MSVCP_logic_error_dtor)
 
 /* out_of_range class data */
 typedef logic_error out_of_range;
 
-static out_of_range* MSVCP_out_of_range_ctor(
-        out_of_range *this, const char **name)
+static out_of_range* MSVCP_out_of_range_ctor( out_of_range *this, exception_name name )
 {
-    TRACE("%p %s\n", this, *name);
+    TRACE("%p %s\n", this, EXCEPTION_STR(name));
     MSVCP_logic_error_ctor(this, name);
     this->e.vtable = &MSVCP_out_of_range_vtable;
     return this;
 }
 
+/* ??0out_of_range@std@@QAE@ABV01@@Z */
+/* ??0out_of_range@std@@QEAA@AEBV01@@Z */
 DEFINE_THISCALL_WRAPPER(MSVCP_out_of_range_copy_ctor, 8)
 out_of_range* __thiscall MSVCP_out_of_range_copy_ctor(
-        out_of_range *this, out_of_range *rhs)
+        out_of_range *this, const out_of_range *rhs)
 {
     TRACE("%p %p\n", this, rhs);
     MSVCP_logic_error_copy_ctor(this, rhs);
@@ -306,16 +414,34 @@ out_of_range* __thiscall MSVCP_out_of_range_copy_ctor(
     return this;
 }
 
+/* ??0out_of_range@std@@QAE@ABV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@1@@Z */
+/* ??0out_of_range@std@@QEAA@AEBV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@1@@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_out_of_range_ctor_bstr, 8)
+out_of_range* __thiscall MSVCP_out_of_range_ctor_bstr(out_of_range *this, const basic_string_char *str)
+{
+    const char *name = MSVCP_basic_string_char_c_str(str);
+    TRACE("(%p %p %s)\n", this, str, name);
+    return MSVCP_out_of_range_ctor(this, EXCEPTION_NAME(name));
+}
+
+/* ??4out_of_range@std@@QAEAAV01@ABV01@@Z */
+/* ??4out_of_range@std@@QEAAAEAV01@AEBV01@@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_out_of_range_assign, 8)
+out_of_range* __thiscall MSVCP_out_of_range_assign(out_of_range *this, const out_of_range *assign)
+{
+    MSVCP_logic_error_dtor(this);
+    return MSVCP_out_of_range_copy_ctor(this, assign);
+}
+
 DEFINE_RTTI_DATA2(out_of_range, 0, &logic_error_rtti_base_descriptor, &exception_rtti_base_descriptor, ".?AVout_of_range@std@@")
 DEFINE_CXX_DATA2(out_of_range, &logic_error_cxx_type_info, &exception_cxx_type_info, MSVCP_logic_error_dtor)
 
 /* invalid_argument class data */
 typedef logic_error invalid_argument;
 
-static invalid_argument* MSVCP_invalid_argument_ctor(
-        invalid_argument *this, const char **name)
+static invalid_argument* MSVCP_invalid_argument_ctor( invalid_argument *this, exception_name name )
 {
-    TRACE("%p %s\n", this, *name);
+    TRACE("%p %s\n", this, EXCEPTION_STR(name));
     MSVCP_logic_error_ctor(this, name);
     this->e.vtable = &MSVCP_invalid_argument_vtable;
     return this;
@@ -340,20 +466,21 @@ typedef struct {
     basic_string_char str;
 } runtime_error;
 
-static runtime_error* MSVCP_runtime_error_ctor(
-        runtime_error *this, const char **name)
+static runtime_error* MSVCP_runtime_error_ctor( runtime_error *this, exception_name name )
 {
-    TRACE("%p %s\n", this, *name);
+    TRACE("%p %s\n", this, EXCEPTION_STR(name));
     this->e.vtable = &MSVCP_runtime_error_vtable;
     this->e.name = NULL;
     this->e.do_free = FALSE;
-    MSVCP_basic_string_char_ctor_cstr(&this->str, *name);
+    MSVCP_basic_string_char_ctor_cstr(&this->str, EXCEPTION_STR(name));
     return this;
 }
 
+/* ??0runtime_error@std@@QAE@ABV01@@Z */
+/* ??0runtime_error@std@@QEAA@AEBV01@@Z */
 DEFINE_THISCALL_WRAPPER(MSVCP_runtime_error_copy_ctor, 8)
 runtime_error* __thiscall MSVCP_runtime_error_copy_ctor(
-        runtime_error *this, runtime_error *rhs)
+        runtime_error *this, const runtime_error *rhs)
 {
     TRACE("%p %p\n", this, rhs);
     MSVCP_exception_copy_ctor(&this->e, &rhs->e);
@@ -362,6 +489,18 @@ runtime_error* __thiscall MSVCP_runtime_error_copy_ctor(
     return this;
 }
 
+/* ??0runtime_error@std@@QAE@ABV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@1@@Z */
+/* ??0runtime_error@std@@QEAA@AEBV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@1@@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_runtime_error_ctor_bstr, 8)
+runtime_error* __thiscall MSVCP_runtime_error_ctor_bstr(runtime_error *this, const basic_string_char *str)
+{
+    const char *name = MSVCP_basic_string_char_c_str(str);
+    TRACE("(%p %p %s)\n", this, str, name);
+    return MSVCP_runtime_error_ctor(this, EXCEPTION_NAME(name));
+}
+
+/* ??1runtime_error@std@@UAE@XZ */
+/* ??1runtime_error@std@@UEAA@XZ */
 DEFINE_THISCALL_WRAPPER(MSVCP_runtime_error_dtor, 4)
 void __thiscall MSVCP_runtime_error_dtor(runtime_error *this)
 {
@@ -391,6 +530,17 @@ void* __thiscall MSVCP_runtime_error_vector_dtor(
     return this;
 }
 
+/* ??4runtime_error@std@@QAEAAV01@ABV01@@Z */
+/* ??4runtime_error@std@@QEAAAEAV01@AEBV01@@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_runtime_error_assign, 8)
+runtime_error* __thiscall MSVCP_runtime_error_assign(runtime_error *this, const runtime_error *assign)
+{
+    MSVCP_runtime_error_dtor(this);
+    return MSVCP_runtime_error_copy_ctor(this, assign);
+}
+
+/* ?what@runtime_error@std@@UBEPBDXZ */
+/* ?what@runtime_error@std@@UEBAPEBDXZ */
 DEFINE_THISCALL_WRAPPER(MSVCP_runtime_error_what, 4)
 const char* __thiscall MSVCP_runtime_error_what(runtime_error *this)
 {
@@ -404,10 +554,9 @@ DEFINE_CXX_DATA1(runtime_error, &exception_cxx_type_info, MSVCP_runtime_error_dt
 /* failure class data */
 typedef runtime_error failure;
 
-static failure* MSVCP_failure_ctor(
-        failure *this, const char **name)
+static failure* MSVCP_failure_ctor( failure *this, exception_name name )
 {
-    TRACE("%p %s\n", this, *name);
+    TRACE("%p %s\n", this, EXCEPTION_STR(name));
     MSVCP_runtime_error_ctor(this, name);
     this->e.vtable = &MSVCP_failure_vtable;
     return this;
@@ -451,21 +600,27 @@ DEFINE_CXX_DATA2(failure, &runtime_error_cxx_type_info, &exception_cxx_type_info
 /* bad_cast class data */
 typedef exception bad_cast;
 
+/* ??0bad_cast@std@@QAE@PBD@Z */
+/* ??0bad_cast@std@@QEAA@PEBD@Z */
 DEFINE_THISCALL_WRAPPER(MSVCP_bad_cast_ctor, 8)
 bad_cast* __thiscall MSVCP_bad_cast_ctor(bad_cast *this, const char *name)
 {
     TRACE("%p %s\n", this, name);
-    MSVCP_exception_ctor(this, &name);
+    MSVCP_exception_ctor(this, EXCEPTION_NAME(name));
     this->vtable = &MSVCP_bad_cast_vtable;
     return this;
 }
 
+/* ??_Fbad_cast@@QAEXXZ */
+/* ??_Fbad_cast@std@@QEAAXXZ */
 DEFINE_THISCALL_WRAPPER(MSVCP_bad_cast_default_ctor,4)
 bad_cast* __thiscall MSVCP_bad_cast_default_ctor(bad_cast *this)
 {
     return MSVCP_bad_cast_ctor(this, "bad cast");
 }
 
+/* ??0bad_cast@std@@QAE@ABV01@@Z */
+/* ??0bad_cast@std@@QEAA@AEBV01@@Z */
 DEFINE_THISCALL_WRAPPER(MSVCP_bad_cast_copy_ctor, 8)
 bad_cast* __thiscall MSVCP_bad_cast_copy_ctor(bad_cast *this, const bad_cast *rhs)
 {
@@ -475,6 +630,8 @@ bad_cast* __thiscall MSVCP_bad_cast_copy_ctor(bad_cast *this, const bad_cast *rh
     return this;
 }
 
+/* ??1bad_cast@@UAE@XZ */
+/* ??1bad_cast@std@@UEAA@XZ */
 DEFINE_THISCALL_WRAPPER(MSVCP_bad_cast_dtor, 4)
 void __thiscall MSVCP_bad_cast_dtor(bad_cast *this)
 {
@@ -502,6 +659,8 @@ void * __thiscall MSVCP_bad_cast_vector_dtor(bad_cast *this, unsigned int flags)
     return this;
 }
 
+/* ??4bad_cast@std@@QAEAAV01@ABV01@@Z */
+/* ??4bad_cast@std@@QEAAAEAV01@AEBV01@@Z */
 DEFINE_THISCALL_WRAPPER(MSVCP_bad_cast_opequals, 8)
 bad_cast* __thiscall MSVCP_bad_cast_opequals(bad_cast *this, const bad_cast *rhs)
 {
@@ -569,38 +728,44 @@ MSVCP_bool __cdecl MSVCP__uncaught_exception(void)
     return __uncaught_exception();
 }
 
+#if _MSVCP_VER >= 70
+#define EXCEPTION_VTABLE(name,funcs) __ASM_VTABLE(name,funcs)
+#else
+#define EXCEPTION_VTABLE(name,funcs) __ASM_VTABLE(name,funcs VTABLE_ADD_FUNC(MSVCP_exception__Doraise))
+#endif
+
 #ifndef __GNUC__
 void __asm_dummy_vtables(void) {
 #endif
     __ASM_VTABLE(type_info,
             VTABLE_ADD_FUNC(MSVCP_type_info_vector_dtor));
-    __ASM_VTABLE(exception,
+    EXCEPTION_VTABLE(exception,
             VTABLE_ADD_FUNC(MSVCP_exception_vector_dtor)
-            VTABLE_ADD_FUNC(MSVCP_what_exception));
-    __ASM_VTABLE(bad_alloc,
+            VTABLE_ADD_FUNC(MSVCP_exception_what));
+    EXCEPTION_VTABLE(bad_alloc,
             VTABLE_ADD_FUNC(MSVCP_bad_alloc_vector_dtor)
-            VTABLE_ADD_FUNC(MSVCP_what_exception));
-    __ASM_VTABLE(logic_error,
+            VTABLE_ADD_FUNC(MSVCP_exception_what));
+    EXCEPTION_VTABLE(logic_error,
             VTABLE_ADD_FUNC(MSVCP_logic_error_vector_dtor)
             VTABLE_ADD_FUNC(MSVCP_logic_error_what));
-    __ASM_VTABLE(length_error,
+    EXCEPTION_VTABLE(length_error,
             VTABLE_ADD_FUNC(MSVCP_logic_error_vector_dtor)
             VTABLE_ADD_FUNC(MSVCP_logic_error_what));
-    __ASM_VTABLE(out_of_range,
+    EXCEPTION_VTABLE(out_of_range,
             VTABLE_ADD_FUNC(MSVCP_logic_error_vector_dtor)
             VTABLE_ADD_FUNC(MSVCP_logic_error_what));
-    __ASM_VTABLE(invalid_argument,
+    EXCEPTION_VTABLE(invalid_argument,
             VTABLE_ADD_FUNC(MSVCP_logic_error_vector_dtor)
             VTABLE_ADD_FUNC(MSVCP_logic_error_what));
-    __ASM_VTABLE(runtime_error,
+    EXCEPTION_VTABLE(runtime_error,
             VTABLE_ADD_FUNC(MSVCP_runtime_error_vector_dtor)
             VTABLE_ADD_FUNC(MSVCP_runtime_error_what));
-    __ASM_VTABLE(failure,
+    EXCEPTION_VTABLE(failure,
             VTABLE_ADD_FUNC(MSVCP_failure_vector_dtor)
             VTABLE_ADD_FUNC(MSVCP_failure_what));
-    __ASM_VTABLE(bad_cast,
+    EXCEPTION_VTABLE(bad_cast,
             VTABLE_ADD_FUNC(MSVCP_bad_cast_vector_dtor)
-            VTABLE_ADD_FUNC(MSVCP_what_exception));
+            VTABLE_ADD_FUNC(MSVCP_exception_what));
 #ifndef __GNUC__
 }
 #endif
@@ -608,49 +773,54 @@ void __asm_dummy_vtables(void) {
 /* Internal: throws selected exception */
 void throw_exception(exception_type et, const char *str)
 {
-    const char *addr = str;
+    exception_name name = EXCEPTION_NAME(str);
 
     switch(et) {
     case EXCEPTION_RERAISE:
         _CxxThrowException(NULL, NULL);
     case EXCEPTION: {
         exception e;
-        MSVCP_exception_ctor(&e, &addr);
+        MSVCP_exception_ctor(&e, name);
         _CxxThrowException(&e, &exception_cxx_type);
     }
     case EXCEPTION_BAD_ALLOC: {
         bad_alloc e;
-        MSVCP_bad_alloc_ctor(&e, &addr);
+        MSVCP_bad_alloc_ctor(&e, name);
         _CxxThrowException(&e, &bad_alloc_cxx_type);
     }
+    case EXCEPTION_BAD_CAST: {
+        bad_cast e;
+        MSVCP_bad_cast_ctor(&e, str);
+        _CxxThrowException(&e, &bad_cast_cxx_type);
+    }
     case EXCEPTION_LOGIC_ERROR: {
         logic_error e;
-        MSVCP_logic_error_ctor(&e, &addr);
+        MSVCP_logic_error_ctor(&e, name);
         _CxxThrowException((exception*)&e, &logic_error_cxx_type);
     }
     case EXCEPTION_LENGTH_ERROR: {
         length_error e;
-        MSVCP_length_error_ctor(&e, &addr);
+        MSVCP_length_error_ctor(&e, name);
         _CxxThrowException((exception*)&e, &length_error_cxx_type);
     }
     case EXCEPTION_OUT_OF_RANGE: {
         out_of_range e;
-        MSVCP_out_of_range_ctor(&e, &addr);
+        MSVCP_out_of_range_ctor(&e, name);
         _CxxThrowException((exception*)&e, &out_of_range_cxx_type);
     }
     case EXCEPTION_INVALID_ARGUMENT: {
         invalid_argument e;
-        MSVCP_invalid_argument_ctor(&e, &addr);
+        MSVCP_invalid_argument_ctor(&e, name);
         _CxxThrowException((exception*)&e, &invalid_argument_cxx_type);
     }
     case EXCEPTION_RUNTIME_ERROR: {
         runtime_error e;
-        MSVCP_runtime_error_ctor(&e, &addr);
+        MSVCP_runtime_error_ctor(&e, name);
         _CxxThrowException((exception*)&e, &runtime_error_cxx_type);
     }
     case EXCEPTION_FAILURE: {
         failure e;
-        MSVCP_failure_ctor(&e, &addr);
+        MSVCP_failure_ctor(&e, name);
         _CxxThrowException((exception*)&e, &failure_cxx_type);
     }
     }
