@@ -71,16 +71,23 @@ extern MSVCP_bool (__thiscall *critical_section_trylock)(critical_section*);
 #endif
 
 /* basic_string<char, char_traits<char>, allocator<char>> */
-#define BUF_SIZE_CHAR 16
 typedef struct
 {
+#if _MSVCP_VER <= 90
     void *allocator;
+#endif
     union {
+#if _MSVCP_VER >= 70
+#define BUF_SIZE_CHAR 16
         char buf[BUF_SIZE_CHAR];
+#endif
         char *ptr;
     } data;
     MSVCP_size_t size;
     MSVCP_size_t res;
+#if _MSVCP_VER == 100
+    char allocator;
+#endif
 } basic_string_char;
 
 basic_string_char* __thiscall MSVCP_basic_string_char_ctor(basic_string_char*);
@@ -94,16 +101,23 @@ basic_string_char* __thiscall MSVCP_basic_string_char_append_ch(basic_string_cha
 MSVCP_size_t __thiscall MSVCP_basic_string_char_length(const basic_string_char*);
 basic_string_char* __thiscall MSVCP_basic_string_char_assign(basic_string_char*, const basic_string_char*);
 
-#define BUF_SIZE_WCHAR 8
 typedef struct
 {
+#if _MSVCP_VER <= 90
     void *allocator;
+#endif
     union {
+#if _MSVCP_VER >= 70
+#define BUF_SIZE_WCHAR 8
         wchar_t buf[BUF_SIZE_WCHAR];
+#endif
         wchar_t *ptr;
     } data;
     MSVCP_size_t size;
     MSVCP_size_t res;
+#if _MSVCP_VER == 100
+    char allocator;
+#endif
 } basic_string_wchar;
 
 basic_string_wchar* __thiscall MSVCP_basic_string_wchar_ctor(basic_string_wchar*);
@@ -117,10 +131,10 @@ MSVCP_size_t __thiscall MSVCP_basic_string_wchar_length(const basic_string_wchar
 
 char* __thiscall MSVCP_allocator_char_allocate(void*, MSVCP_size_t);
 void __thiscall MSVCP_allocator_char_deallocate(void*, char*, MSVCP_size_t);
-MSVCP_size_t __thiscall MSVCP_allocator_char_max_size(void*);
+MSVCP_size_t __thiscall MSVCP_allocator_char_max_size(const void*);
 wchar_t* __thiscall MSVCP_allocator_wchar_allocate(void*, MSVCP_size_t);
 void __thiscall MSVCP_allocator_wchar_deallocate(void*, wchar_t*, MSVCP_size_t);
-MSVCP_size_t __thiscall MSVCP_allocator_wchar_max_size(void*);
+MSVCP_size_t __thiscall MSVCP_allocator_wchar_max_size(const void*);
 
 typedef struct
 {
@@ -240,7 +254,11 @@ ctype_wchar* ctype_short_use_facet(const locale*);
 
 /* class _Lockit */
 typedef struct {
+#if _MSVCP_VER >= 70
     int locktype;
+#else
+    char empty_struct;
+#endif
 } _Lockit;
 
 #define _LOCK_LOCALE 0
@@ -342,7 +360,7 @@ typedef struct _ios_base {
     const vtable_ptr *vtable;
 #if _MSVCP_VER >= 100
     MSVCP_size_t DECLSPEC_ALIGN(8) stdstr;
-#else
+#elif _MSVCP_VER >= 71
     MSVCP_size_t stdstr;
 #endif
     IOSB_iostate state;
@@ -352,13 +370,20 @@ typedef struct _ios_base {
     streamsize wide;
     IOS_BASE_iosarray *arr;
     IOS_BASE_fnarray *calls;
+#if _MSVCP_VER < 70
+    locale loc;
+#else
     locale *loc;
+#endif
+#if _MSVCP_VER <= 70
+    MSVCP_size_t stdstr;
+#endif
 } ios_base;
 
 /* class basic_streambuf<char> */
 typedef struct {
     const vtable_ptr *vtable;
-#if _MSVCP_VER <= 100
+#if _MSVCP_VER >= 70 && _MSVCP_VER <= 100
     mutex lock;
 #endif
     char *rbuf;
@@ -373,7 +398,11 @@ typedef struct {
     int wsize;
     int *prsize;
     int *pwsize;
+#if _MSVCP_VER < 70
+    locale loc;
+#else
     locale *loc;
+#endif
 } basic_streambuf_char;
 
 typedef struct {
@@ -397,7 +426,7 @@ int __thiscall basic_streambuf_char_sputc(basic_streambuf_char*, char);
 /* class basic_streambuf<wchar> */
 typedef struct {
     const vtable_ptr *vtable;
-#if _MSVCP_VER <= 100
+#if _MSVCP_VER >= 70 && _MSVCP_VER <= 100
     mutex lock;
 #endif
     wchar_t *rbuf;
@@ -412,7 +441,11 @@ typedef struct {
     int wsize;
     int *prsize;
     int *pwsize;
+#if _MSVCP_VER < 70
+    locale loc;
+#else
     locale *loc;
+#endif
 } basic_streambuf_wchar;
 
 typedef struct {
@@ -433,6 +466,12 @@ unsigned short __thiscall basic_streambuf_wchar_sgetc(basic_streambuf_wchar*);
 unsigned short __thiscall basic_streambuf_wchar_sbumpc(basic_streambuf_wchar*);
 unsigned short __thiscall basic_streambuf_wchar_sputc(basic_streambuf_wchar*, wchar_t);
 
+#if _MSVCP_VER < 70
+#define IOS_LOCALE(ios) (&(ios)->loc)
+#else
+#define IOS_LOCALE(ios) ((ios)->loc)
+#endif
+
 /* class num_get<char> */
 typedef struct {
     locale_facet facet;
@@ -536,6 +575,7 @@ void init_exception(void*);
 void init_locale(void*);
 void init_io(void*);
 void free_io(void);
+void init_misc(void*);
 
 /* class complex<float> */
 typedef struct {
@@ -551,8 +591,23 @@ typedef struct {
 } complex_double;
 
 #if _MSVCP_VER < 80
-#define memcpy_s( dst, size, src, count ) (memcpy( (dst), (src), (count) ), 0)
-#define memmove_s( dst, size, src, count ) (memmove( (dst), (src), (count) ), 0)
-#define mbstowcs_s( ret, wcs, size, mbs, count ) (mbstowcs( (wcs), (mbs), (count) ), 0)
+static inline int memcpy_wrapper( void *dst, size_t size, const void *src, size_t count )
+{
+    memcpy( dst, src, count );
+    return 0;
+}
+static inline int memmove_wrapper( void *dst, size_t size, const void *src, size_t count )
+{
+    memmove( dst, src, count );
+    return 0;
+}
+static inline int mbstowcs_wrapper( size_t *ret, wchar_t *wcs, size_t size, const char *mbs, size_t count )
+{
+    mbstowcs( wcs, mbs, count );
+    return 0;
+}
+#define memcpy_s( dst, size, src, count ) memcpy_wrapper( dst, size, src, count )
+#define memmove_s( dst, size, src, count ) memmove_wrapper( dst, size, src, count )
+#define mbstowcs_s( ret, wcs, size, mbs, count ) mbstowcs_wrapper( ret, wcs, size, mbs, count )
 #define hypotf( x, y ) ((float)hypot( (double)(x), (double)(y) ))
 #endif
