@@ -932,7 +932,7 @@ START_TEST(winstation)
     GetProcessWindowStation();
     if (GetLastError() == ERROR_CALL_NOT_IMPLEMENTED)
     {
-        skip("WindowStation calls not supported on this platform\n");
+        win_skip("WindowStation calls not supported on this platform\n");
         return;
     }
 
