@@ -219,11 +219,16 @@ struct thread *create_thread( int fd, struct process *process )
 
     if (process->is_terminating)
     {
+        close( fd );
         set_error( STATUS_PROCESS_IS_TERMINATING );
         return NULL;
     }
 
-    if (!(thread = alloc_object( &thread_ops ))) return NULL;
+    if (!(thread = alloc_object( &thread_ops )))
+    {
+        close( fd );
+        return NULL;
+    }
 
     init_thread_structure( thread );
 
@@ -236,6 +241,7 @@ struct thread *create_thread( int fd, struct process *process )
 
     if (!(thread->id = alloc_ptid( thread )))
     {
+        close( fd );
         release_object( thread );
         return NULL;
     }
@@ -1047,6 +1053,8 @@ int thread_add_inflight_fd( struct thread *thread, int client, int server )
             thread->inflight[i].server = server;
             return i;
         }
+
+    close( server );
     return -1;
 }
 
@@ -1443,8 +1451,8 @@ DECL_HANDLER(select)
         else if (apc->result.type == APC_ASYNC_IO)
         {
             if (apc->owner)
-                async_set_result( apc->owner, apc->result.async_io.status,
-                                  apc->result.async_io.total, apc->result.async_io.apc );
+                async_set_result( apc->owner, apc->result.async_io.status, apc->result.async_io.total,
+                                  apc->result.async_io.apc, apc->result.async_io.arg );
         }
         wake_up( &apc->obj, 0 );
         close_handle( current->process, req->prev_apc );
